"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlainArrayToClassTest = void 0;
const mocha_1 = require("@testdeck/mocha");
const chai_1 = require("chai");
const src_1 = require("../src");
const UnknownClassError_1 = require("./../src/exceptions/UnknownClassError");
const CustomStorageClass_1 = require("./classes/CustomStorageClass");
const Override_1 = require("./classes/Override");
const PassClassTransformOption_1 = require("./classes/PassClassTransformOption");
const PersonalBlog_1 = require("./classes/PersonalBlog");
const Product_1 = require("./classes/Product");
const Ship_1 = require("./classes/Ship");
const factories_1 = require("./factories");
let PlainArrayToClassTest = class PlainArrayToClassTest {
    normal() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        Product_1.productValidate(expected, src_1.plainArrayToClass(Product_1.Product, expected.toPlainArray()));
    }
    array() {
        const testData = factories_1.factory.make(Product_1.Product).many(2);
        const results = src_1.plainArrayToClass(Product_1.Product, testData.map((o) => o.toPlainArray()), { isArray: true });
        chai_1.expect(results).property('constructor', Array);
        chai_1.expect(results).length(testData.length);
        for (const [i, expected] of testData.entries()) {
            chai_1.expect(results).property(i.toString());
            Product_1.productValidate(expected, results[i]);
        }
    }
    classTransformOption() {
        const expected = factories_1.factory.make(PassClassTransformOption_1.PassClassTransformOption).one();
        const result = src_1.plainArrayToClass(PassClassTransformOption_1.PassClassTransformOption, expected.toPlainArray(), { strategy: 'excludeAll' });
        PassClassTransformOption_1.passClassTransformOptionValidate(expected, result);
    }
    extendsClass() {
        const expected = factories_1.factory.make(PersonalBlog_1.PersonalBlog).one();
        PersonalBlog_1.personalBlogValidate(expected, src_1.plainArrayToClass(PersonalBlog_1.PersonalBlog, expected.toPlainArray()));
    }
    overrideClass() {
        const expected = factories_1.factory.make(Override_1.Override).one();
        Override_1.overrideValidate(Object.assign(new Override_1.Override(), {
            weight: expected.weight,
        }), src_1.plainArrayToClass(Override_1.Override, expected.toPlainArray()));
    }
    customStorage() {
        const expected = factories_1.factory.make(CustomStorageClass_1.CustomStorageClass).one();
        CustomStorageClass_1.customStorageValidate(expected, src_1.plainArrayToClass(CustomStorageClass_1.CustomStorageClass, expected.toPlainArray(), { arrayMemberStorage: CustomStorageClass_1.arrayMemberStorage }));
    }
    customStorageFail() {
        const expected = factories_1.factory.make(CustomStorageClass_1.CustomStorageClass).one();
        chai_1.expect(() => src_1.plainArrayToClass(CustomStorageClass_1.CustomStorageClass, expected.toPlainArray())).throw(UnknownClassError_1.UnknownClassError);
    }
    customStorageDecorator() {
        const expected = factories_1.factory.make(Ship_1.Ship).one();
        Ship_1.shipValidate(expected, src_1.plainArrayToClass(Ship_1.Ship, expected.toPlainArray(), { arrayMemberStorage: Ship_1.shipStorage }));
    }
    customStorageDecoratorFail() {
        const expected = factories_1.factory.make(Ship_1.Ship).one();
        chai_1.expect(() => src_1.plainArrayToClass(Ship_1.Ship, expected.toPlainArray())).throw(UnknownClassError_1.UnknownClassError);
    }
    customStorageDecoratorWithDefault() {
        const expected = factories_1.factory.make(Ship_1.ShipWithDefault).one();
        Ship_1.shipWithDefaultValidate(expected, src_1.plainArrayToClass(Ship_1.ShipWithDefault, expected.toPlainArray(), { arrayMemberStorage: Ship_1.shipStorage }));
        Ship_1.shipWithDefaultValidate(expected, src_1.plainArrayToClass(Ship_1.ShipWithDefault, expected.toPlainArray()));
    }
    customStorageDecoratorPartial() {
        const expected = factories_1.factory.make(Ship_1.ShipWithPartialProperty).one();
        Ship_1.shipWithPartialPropertyValidate(Object.assign(new Ship_1.ShipWithPartialProperty(), { ...expected, name: '' }), src_1.plainArrayToClass(Ship_1.ShipWithPartialProperty, expected.toPlainArray(), { arrayMemberStorage: Ship_1.shipStorage }));
        Ship_1.shipWithPartialPropertyValidate(Object.assign(new Ship_1.ShipWithPartialProperty(), { ...expected, id: 0 }), src_1.plainArrayToClass(Ship_1.ShipWithPartialProperty, expected.toPlainArray(), { arrayMemberStorage: Ship_1.shipStoragePartial }));
    }
    customStorageDecoratorPartialFail() {
        const expected = factories_1.factory.make(Ship_1.ShipWithPartialProperty).one();
        chai_1.expect(() => src_1.plainArrayToClass(Ship_1.ShipWithPartialProperty, expected.toPlainArray())).throw(UnknownClassError_1.UnknownClassError);
    }
};
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "normal", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "array", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "classTransformOption", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "extendsClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "overrideClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorage", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageFail", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageDecorator", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageDecoratorFail", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageDecoratorWithDefault", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageDecoratorPartial", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "customStorageDecoratorPartialFail", null);
PlainArrayToClassTest = __decorate([
    mocha_1.suite()
], PlainArrayToClassTest);
exports.PlainArrayToClassTest = PlainArrayToClassTest;
//# sourceMappingURL=plainArrayToClass.test.js.map